/*:ja
 * @target MZ
 *
 * @param ButtonList
 * @text ボタンリスト
 * @desc ボタンのリストです。
 * @default []
 * @type struct<Button>[]
 * 
 * @param VisileSw
 * @text 全体表示オンオフスイッチ
 * @type switch
 * @default 0
 * 
 * @command create
 * @text ボタン生成
 *
 * @arg params
 * @type struct<Button>
 *
 * @command hide
 * @arg btnName
 * @type string
 * 
 * @command show
 * @arg btnName
 * @type string
 * 
 * @command enable
 * @arg btnName
 * @type string
 * 
 * @command disable
 * @arg btnName
 * @type string
 * 
 */

/*~struct~Button:
*
* @param btnName
* @type string
*
* @param commonEventId
* @type common_event
* @default 0
*  
* @param picName
* @type file
* @dir img/pictures/
* 
* @param x
* @type number
* @default 0
* 
* @param y
* @type number
* @default 0
* 
* @param condition
* @type string
* 
* @param mapIds
* @type number[]
* @default []
*/

$btns = null;

(() => {
    const pluginName = "Button";
    const script = document.currentScript;
    const param = PluginManagerEx.createParameter(script);

    PIXI.DisplayObject.prototype.isChildrenBeingTouched = function () {
        let a = this.children.some(a => a.isChildrenBeingTouched())
        let b = false
        if (this.isBeingTouched) {
            b = this.isBeingTouched() && this.onClick != Sprite_Clickable.prototype.onClick
        } else if (this.isTouchedInsideFrame) {
            b = this.isTouchedInsideFrame() && this.isOpen()
        }
        return a || (b && this.worldVisible && !this.isThrough)
    }

    const Sprite_Clickable_isClickEnabled = Sprite_Clickable.prototype.isClickEnabled
    Sprite_Clickable.prototype.isClickEnabled = function () {

        return SceneManager._scene?._menuButton?.visible && Sprite_Clickable_isClickEnabled.apply(this, arguments);
    };


    class SpriteBtn extends Sprite_Clickable {
        constructor(btnName, picName, x, y, commonEventId, condition, mapIds) {
            super()
            this.bitmap = ImageManager.loadPicture(picName)
            this.name = btnName;
            this.commonEventId = commonEventId;
            this.x = x;
            this.y = y;
            this.enabled = true
            this.condition = condition ? new Function("return " + condition) : undefined
            this.mapIds = mapIds
        }
        static createBtns() {
            let btns = new Sprite()
            param.ButtonList.forEach(args => {
                let btn = new SpriteBtn(args.btnName, args.picName, args.x, args.y, args.commonEventId, args.condition, args.mapIds)
                btns.addChild(btn);
            });
            return btns
        }
        static GetByName(btnName) {
            return $btns.getChildByName(btnName)
        }
        onClick() {
            if (this.commonEventId && this.commonEventId > 0)
                $gameTemp.reserveCommonEvent(this.commonEventId);
        }
        onMouseEnter() {
            this.opacity = 180;
        }
        onMouseExit() {
            this.opacity = 255;
        }
        enable() {
            this.enabled = true
            this.opacity = 255
        }
        disable() {
            this.enabled = false
            this.opacity = 30
        }
        isClickEnabled() {
            return this.visible && this.enabled;
        }
        update() {
            super.update()
            if (this.condition) {
                if (this.condition()) {
                    if (!this.enabled) this.enable()
                } else {
                    if (this.enabled) this.disable()
                }
            }
            this.visible = s(param.VisileSw) &&
                (this.mapIds.contains($gameMap.mapId()) || this.mapIds.length == 0)
        }
    }

    PluginManagerEx.registerCommand(script, "hide", args => {
        SpriteBtn.GetByName(args.btnName).hide()
    });

    PluginManagerEx.registerCommand(script, "show", args => {
        SpriteBtn.GetByName(args.btnName).show()
    });
    PluginManagerEx.registerCommand(script, "enable", args => {
        SpriteBtn.GetByName(args.btnName).enable()
    });
    PluginManagerEx.registerCommand(script, "disable", args => {
        SpriteBtn.GetByName(args.btnName).disable()
    });

    const _Scene_Map_isAnyButtonPressed = Scene_Map.prototype.isAnyButtonPressed;
    Scene_Map.prototype.isAnyButtonPressed = function () {
        return (
            _Scene_Map_isAnyButtonPressed.apply(this, arguments) || this.isChildrenBeingTouched()
        );
    };


    const SceneManager_onBeforeSceneStart = SceneManager.onBeforeSceneStart
    SceneManager.onBeforeSceneStart = function () {
        if (this._previousScene) {
            this._previousScene.removeChild($btns)
        }
        SceneManager_onBeforeSceneStart.apply(this, arguments);
    }
    const _Scene_Map_start = Scene_Map.prototype.start;
    Scene_Map.prototype.start = function () {
        _Scene_Map_start.apply(this, arguments);
        if ($btns == null)
            $btns = SpriteBtn.createBtns();
        SceneManager._scene.addChild($btns)
    };


    ImageManager.clear = function () {

    }
})();
