/*:ja
 * @target MZ
 *
 * @param common
 * @type common_event
 * 
 * @command toggle
 * @command toggleGarally
 * 
 * @command create
 * 
 * @param charaVar
 * @type variable
 * 
 * @param onCamera
 * @type common_event
 * 
 * */
var $smartPhone = null;

(() => {
    'use strict';

    const script = document.currentScript;
    const param = PluginManagerEx.createParameter(script);

    const STD_SCALE = 0.97
    const ZOOM_SCALE = 1.5
    const SCREE_RECT_WIDT_HOME = new Rectangle(16, 49, 307, 549 + 44)
    const HOME_HEIGHT = 44
    const SCREEN_RECT = new Rectangle(SCREE_RECT_WIDT_HOME.x, SCREE_RECT_WIDT_HOME.y, SCREE_RECT_WIDT_HOME.width, SCREE_RECT_WIDT_HOME.height - HOME_HEIGHT)
    const X = Math.floor(720 / 2) - (339 / 2)
    const Y = 0


    class SmartPhone extends Sprite {
        constructor() {
            super()
            this.name = "SmartPhone"
            this.x = X
            this.y = Y
            this.visible = true
            this.bitmap = ImageManager.loadSystem("SmartPhone/frame");

            this.home = this.addChild(new Home());
            this.home.addApp("SmartPhone/camera", () => { this.set(this.cameraScreen) })
            this.home.addApp("SmartPhone/gallery", () => { this.set(this.gallery) })
            this.home.addApp("SmartPhone/line", () => { this.set(this.line) })
            this.cameraScreen = this.addChild(new CameraScreen())
            this.gallery = this.addChild(new Window_ScreenshotGallery())
            this.line = this.addChild(new Window_Line())
            this.menu = this.addChild(new Menu());
            this.menu.homeBtn.onClick = () => { this.set(this.home) }
            this.menu.backBtn.onClick = () => { this.pop() }
            this.set(this.cameraScreen)
        }
        default() {
            this.set(this.home)
            this.visible = false
        }
        set(current) {
            this.current = current
            this.children.forEach(element => {
                element.hide()
            });
            this.menu.show()
            current.show()
            if (current == this.home)
                this.menu.backBtn.hide()
            else
                this.menu.backBtn.show()
        }
        pop() {
            if (this.current.pop)
                this.current.pop()
            else
                this.set(this.home)
        }
        update() {
            super.update()
        }
        toggle() {

        }

    }

    class CameraScreen extends Sprite_Clickable {
        constructor() {
            super()
            this.x = SCREEN_RECT.x, this.y = SCREEN_RECT.y, this.width = SCREEN_RECT.width, this.height = SCREEN_RECT.height
            this.renderTexture = PIXI.RenderTexture.create(Graphics.width, Graphics.height);
            this.cameraScale = STD_SCALE
            this.centerX = Graphics.width / 2
            this.centerY = Graphics.height / 2

            this.screen = this.addChild(new PIXI.Sprite(this.renderTexture))

            this.screen.mask = new PIXI.Graphics();
            this.screen.mask.beginFill();
            let p = this.toGlobal(new Point(0, 0))
            this.screen.mask.drawRect(Graphics.width / 2 - SCREEN_RECT.width / 2, p.y, SCREEN_RECT.width, SCREEN_RECT.height);
            this.screen.mask.endFill();
            //window.a = this.screen

            this.screen.filters = [new PIXI.filters.OldFilmFilter()]
            this.screen.filters[0].noise = 0.2; //ノイズの大きさ
            this.screen.filters[0].seed = 0.5; //ノイズのランダム度合い
            this.screen.filters[0].sepia = 0;
            this.screen.filters[0].vignettingAlpha = 0.5;
            this.screen.filters[0].vignetting = 0.01;
            this.screen.filters[0].scratch = 0
            this.screen.filters[0].vignettingBlur = 0.3

            this.captureBtn = this.addChild(new Sprite_Clickable())
            this.captureBtn.bitmap = ImageManager.loadSystem("SmartPhone/shutter")
            this.captureBtn.x = SCREEN_RECT.width / 2
            this.captureBtn.y = SCREEN_RECT.height - 80
            this.captureBtn.anchor.x = 0.5
            this.captureBtn.onClick = () => {
                this.saveCapture()
            }

            this.touchEnabledBtn = this.addChild(new Sprite_Clickable())
            this.touchEnabledBtn.bitmap = ImageManager.loadSystem("SmartPhone/touch")
            this.touchEnabledBtn.x = 10
            this.touchEnabledBtn.y = SCREEN_RECT.height - 48 - 10
            this.touchEnabledBtn.onClick = () => {
                this.setTouchEnabled(!this.touchEnabled)
                TouchInput.clear()
            }
            this.setTouchEnabled(false)

            // this.rotateBtn = this.addChild(new Sprite_Clickable())
            // this.rotateBtn.bitmap = ImageManager.loadSystem("SmartPhone/rotate")
            // this.rotateBtn.x = this.width - 38 - 10
            // this.rotateBtn.y = SCREEN_RECT.height - 38 - 10
            // this.rotateBtn.onClick = () => {
            //     this.isBackCamera = !this.isBackCamera
            // }
            this.isBackCamera = true
            // this.mob = ImageManager.loadSystem("SmartPhone/mob")
        }
        setTouchEnabled(value) {
            this.touchEnabled = value
            if (value) {
                this.isThrough = true
                this.touchEnabledBtn.opacity = 255
            } else {
                this.touchEnabledBtn.opacity = 100
                this.isThrough = false
            }
        }
        onClick() {
            if (!this.touchEnabled) {
                if (this.cameraScale == STD_SCALE) {
                    this.cameraScale = ZOOM_SCALE
                    this.centerX = TouchInput.x
                    this.centerY = TouchInput.y
                } else {
                    this.cameraScale = STD_SCALE
                    this.centerX = Graphics.width / 2
                    this.centerY = Graphics.height / 2
                }
            }
        }

        update() {
            super.update()
            if (!this.worldVisible) {
                this.prevWorldVisible = false
                return
            }

            if (!this.prevWorldVisible) {
                this.prevWorldVisible = true
                //カメラがオンになった時
            }

            if (this.isBackCamera)
                this.drawCapture()
            else
                this.drawMob()

            if (this.screen.tint < 0xffffff)
                this.screen.tint += 0x111111

            this.screen.filters[0].seed += 0.002;
        }
        drawCapture() {
            SceneManager.snapPictures(this.renderTexture, this.cameraScale, this.centerX, this.centerY)
            let p = this.toGlobal(new Point(0, 0))
            this.screen.x = -p.x;
            this.screen.y = -p.y;
        }
        saveCapture() {
            SceneManager._captureBitmap = new Bitmap(SCREEN_RECT.width, SCREEN_RECT.height);

            // this.screen._texture.frame = new Rectangle(200, 0, 300, 700)
            const canvas = Graphics.app.renderer.extract.canvas(this.renderTexture);
            let p = this.toGlobal(new Point(0, 0))
            // this.screen.bitmap.context.drawImage(canvas, p.x, p.y, SCREEN_RECT.width, SCREEN_RECT.height, 0, 0, SCREEN_RECT.width, SCREEN_RECT.height);
            // this.screen.bitmap.baseTexture.update();
            SceneManager._captureBitmap.context.drawImage(canvas, p.x, p.y, SCREEN_RECT.width, SCREEN_RECT.height, 0, 0, SCREEN_RECT.width, SCREEN_RECT.height);
            SceneManager._captureBitmap.baseTexture.update();
            canvas.width = 0;
            canvas.height = 0;


            //SceneManager._captureBitmap.blt(this.screen.bitmap, 0, 0, SCREEN_RECT.width, SCREEN_RECT.height, 0, 0, SCREEN_RECT.width, SCREEN_RECT.height)
            SceneManager.saveCapture("image_%1_%2")

            AudioManager.playSe({ "name": "all/camera", "volume": 100, "pitch": 100, "pan": 0 })
            this.screen.tint = 0
            TouchInput.clear()
            $gameTemp.reserveCommonEvent(param.onCamera);
        }
        // drawMob() {
        //     this.screen.bitmap.blt(this.mob, 0, 0, SCREEN_RECT.width, SCREEN_RECT.height, 0, 0, SCREEN_RECT.width, SCREEN_RECT.height);
        // }
    }
    class Home extends Sprite_Clickable {
        constructor() {
            super()
            this.onClick = () => { }
            this.iconWidth = 48
            this.maxCols = 5
            this.padding = Math.floor((SCREEN_RECT.width - this.iconWidth * this.maxCols) / this.maxCols)
            this.bitmap = ImageManager.loadSystem("SmartPhone/wall")
            this.setFrame(0, 0, SCREEN_RECT.width, SCREEN_RECT.height)
            this.x = SCREEN_RECT.x, this.y = SCREEN_RECT.y, this.width = SCREEN_RECT.width, this.height = SCREEN_RECT.height
            this.apps = []
        }
        addApp(pic, onClick) {
            let icon = new Sprite_Clickable();
            icon.bitmap = ImageManager.loadSystem(pic)
            icon.onClick = onClick
            icon.x = this.padding + (this.iconWidth + this.padding) * (this.apps.length % this.maxCols)
            icon.y = this.padding + (this.iconWidth + this.padding) * Math.floor(this.apps.length / this.maxCols)
            this.apps.push(icon)
            this.addChild(icon)
        };

    }
    class Menu extends Sprite_Clickable {
        constructor() {
            super()
            this.onClick = () => { }
            this.x = SCREEN_RECT.x, this.y = SCREEN_RECT.y + SCREEN_RECT.height, this.width = SCREEN_RECT.width, this.height = HOME_HEIGHT

            this.backBtn = this.addBtn("back")
            this.backBtn.x -= 70
            this.homeBtn = this.addBtn("home")

        }
        addBtn(name) {
            let btn = this.addChild(new Sprite_Clickable())
            btn.bitmap = ImageManager.loadSystem("SmartPhone/" + name)
            btn.anchor.x = 0.5
            btn.anchor.y = 0.5
            btn.x = this.width / 2
            btn.y = this.height / 2
            return btn
        }
    }
    class Window_Selectable_Smartphone extends Window_Selectable {
        constructor() {
            super(SCREEN_RECT);
            this.padding = 0
            this.back = this.addChildToBack(new PIXI.Sprite(PIXI.Texture.WHITE));
            this.back.width = SCREEN_RECT.width;
            this.back.height = SCREEN_RECT.height;
            this.items = [];
            this.setHandler('ok', () => { this.openLargeImage() })
            this.createLargeImage()
            this.refresh();
            this.activate();
        }
        show() {
            super.show()
            this.refresh()
            this.open();
            this.activate();
            this.large.hide()
        }
        hide() {
            this.large.hide()
            this.close()
            super.hide()
        }

        itemPadding() {
            return 0;
        }
        colSpacing() {
            return 0;
        }
        rowSpacing() {
            return 1;
        }
        contentsWidth() {
            return this.width
        }
        currentItem() {
            return this.items[this.index()];
        }
        _refreshBack() { }
        _refreshFrame() { }
        playOkSound() {
        };
        refresh() {
            this.items = [];
            this.makeItemList();
            if (this.maxItems() < 1)
                this.deactivate()
            super.refresh();
        }
    }

    class Window_ScreenshotGallery extends Window_Selectable_Smartphone {

        openLargeImage() {
            this.deactivate()
            this.large.bitmap = this.currentItem()
            this.large.setFrame(0, 0, this.width, this.height)
            this.large.show();
            this.pop = () => {
                this.large.onClick()
            }
        }
        createLargeImage() {
            this.large = new Sprite_Clickable();
            this.large.onClick = () => {
                this.large.hide();
                this.activate();
                this.pop = undefined
            }
            this.addChild(this.large);
        }
        maxCols() {
            return 2;
        }
        maxItems() {
            return ImageManager.validScreenshotCount();
        }
        itemHeight() {
            return this.width / 2
        }
        itemWidth() {
            return this.width / 2;
        }

        drawItem(index) {
            if (this.items[index]) {
                const rect = this.itemRect(index);
                const bitmap = this.items[index];
                this.contents.blt(bitmap, 0, (bitmap.height - bitmap.width) / 2, bitmap.width, bitmap.width, rect.x, rect.y, rect.width, rect.height);
            }
        }


        makeItemList() {
            this.items = ImageManager.loadAllScreenshot();
            this.items.find((image) => !image.isReady())?.addLoadListener(() => this.refresh());
        }

    }
    class Window_Line extends Window_Selectable_Smartphone {

        openLargeImage() {
            this.large.callBtn.bitmap = ImageManager.loadSystem("SmartPhone/call1")
            this.deactivate()
            this.large.icon.bitmap = this.currentItem()
            this.large.bitmap.fillAll("#000000")
            this.large.bitmap.drawText($gameActors.actor(this.currentItem().actorId)?.name(), 0, this.width, this.width, this.large.bitmap.fontSize, "center")
            this.large.show();
            this.pop = () => {
                this.large.hide();
                this.activate();
                this.pop = undefined
            }
        }
        createLargeImage() {
            this.large = this.addChild(new Sprite());
            this.large.bitmap = new Bitmap(this.width, this.height)
            this.large.bitmap.fillAll("#000000")
            this.large.bitmap.fontSize = 20
            //  this.large.bitmap.drawText($gameActors.actor(this.index + 1)?.name(), 0, 0)

            this.large.icon = this.large.addChild(new Sprite());
            this.large.icon.x = this.width / 2
            this.large.icon.y = this.width / 2
            this.large.icon.anchor.set(0.5)

            this.large.callBtn = this.large.addChild(new Sprite_Clickable());
            this.large.callBtn.bitmap = ImageManager.loadSystem("SmartPhone/call1")
            this.large.callBtn.anchor.set(0.5)
            this.large.callBtn.x = this.width / 2
            this.large.callBtn.y = this.height - 100
            this.large.callBtn.onClick = () => {
                this.large.callBtn.bitmap = ImageManager.loadSystem("SmartPhone/call2")
                v(param.charaVar, this.currentItem().actorId)
                $gameTemp.reserveCommonEvent(param.common);
            }
        }

        maxCols() {
            return 1;
        }
        maxItems() {
            return $gameSwitches._data.slice(11, 14).filter(a => a).length
        }
        itemHeight() {
            return 80
        }
        itemWidth() {
            return this.width
        }
        rowSpacing() {
            return 0;
        }
        drawItem(index) {
            if (this.items[index]) {
                this.resetFontSettings()
                const padding = 10
                const rect = this.itemRect(index);
                const bitmap = this.items[index];
                this.contents.fillRect(rect.x, rect.y, rect.width, rect.height, "#FFFFFF")
                this.contents.blt(bitmap, 0, 0, bitmap.width, bitmap.height, rect.x + padding, rect.y + padding, this.itemHeight() - padding, this.itemHeight() - padding);
                this.changeOutlineColor("rgba(0, 0, 0, 0.0)");
                this.changeTextColor("#000000")
                this.makeFontSmaller()
                this.drawText($gameActors.actor(this.items[index].actorId)?.name(), rect.x + this.itemHeight() + padding, rect.y + padding)
                this.changeTextColor("#aaaaaa")
                this.makeFontSmaller()
                this.drawText($gameActors.actor(this.items[index].actorId)?.profile(), rect.x + this.itemHeight() + padding, rect.y + 30)
            }
        }


        makeItemList() {
            $gameActors._data.forEach(element => {
                if (s(10 + element._actorId)) {
                    let bitmap = ImageManager.loadPicture(element._actorId + "/profile")
                    bitmap.actorId = element._actorId
                    this.items.push(bitmap)
                }
                //log(element._actorId)
            });

            //this.items.find((image) => !image.isReady())?.addLoadListener(() => this.refresh());
        }

    }
    // const SceneManager_onBeforeSceneStart = SceneManager.onBeforeSceneStart
    // SceneManager.onBeforeSceneStart = function () {
    //     if (this._previousScene) {
    //         this._previousScene.removeChild($smartPhone)
    //     }
    //     SceneManager_onBeforeSceneStart.apply(this, arguments);
    // }
    // const _Scene_Map_start = Scene_Map.prototype.start;
    // Scene_Map.prototype.start = function () {
    //     _Scene_Map_start.apply(this, arguments);
    //     if ($smartPhone == null)
    //         $smartPhone = new SmartPhone()

    //     SceneManager._scene.addChildAt($smartPhone, 1);
    // };
    SceneManager.snapPictures = function (renderTexture, cameraScale, x, y) {
        let container = SceneManager._scene._spriteset._pictureContainer
        container.scale.x = cameraScale
        container.scale.y = cameraScale
        container.x = Math.round(-x * (cameraScale - 1))
        container.y = Math.round(-y * (cameraScale - 1))
        Graphics.app.renderer.render(container, renderTexture);

        container.scale.x = container.scale.y = 1
        container.x = container.y = 0
    };


    function toggle() {
        let s = SceneManager._scene.getChildByName("SmartPhone")
        if (s) {
            s.destroy()
            SceneManager._scene.removeChild(s);
        } else {
            SceneManager._scene.addChildAt(new SmartPhone(), 1);
        }
    }

    PluginManagerEx.registerCommand(script, "toggle", args => {
        toggle()
    });
    PluginManagerEx.registerCommand(script, "create", args => {
        // if ($smartPhone == null)
        //     $smartPhone = 


    });


})();
