/*:ja
 * @target MZ
 *
 * @param length
 * @type number
 * 
 *
 * @command hide
 * 
 * 
 * @command show
 * 
 * @command clear
 * 
 * @command set
 * 
 * @arg slot
 * @type number
 * 
 * @arg classId
 * @type class
 * 
 * @command use
 * 
 * @arg slot
 * @type number
 * 
 */




var $slot = null;


(() => {
    const script = document.currentScript;
    const param = PluginManagerEx.createParameter(script);

    const SLOT_WIDTH = 60;
    const SLOT_RADIUS = 10;
    const SLOT_MARGIN = 10;
    const FRAME_COLOR = 0x000000;
    const FRAME_ALPHA = 0.3;
    const LINE_COLOR = 0xF0E68C;
    const LINE_WEIGHT = 2;




    class SlotBar extends Sprite_Clickable {
        constructor(length) {
            super()
            this.y = Graphics.height - SLOT_MARGIN - SLOT_WIDTH
            this.length = length
            this.x = Math.floor(Graphics.width / 2) - ((this.length * (SLOT_WIDTH + SLOT_MARGIN) - SLOT_MARGIN) / 2);
            for (let i = 0; i < length; i++) {
                this.addChild(new Slot(i))
            }
        }
        getActive() {
            return this.children.find((a) => {
                return a.focus.visible
            })
        }
        clear() {
            this.children.forEach((a) => {
                a.unset()
            })
        }
        update() {
            super.update()
            // this.visible = s(param.VisileSw)
        }
        set(slotNo, classId) {
            $slot.children[slotNo - 1].set(classId)
        }
        setByName(slotNo, name) {
            let c = $dataClasses.find(a => a?.name == name)
            this.set(slotNo, c.id)
        }
        use(slotNo) {
            if (slotNo > 0)
                $slot.children[slotNo - 1].use()
            else {
                $slot.children.forEach(element => {
                    element.unuse()
                });
            }
        }

    }
    class Slot extends Sprite_Clickable {
        constructor(id) {
            super()
            this.id = id + 1;
            this.x = id * (SLOT_WIDTH + SLOT_MARGIN)
            this.width = SLOT_WIDTH
            this.class = null
            this.createForcus()
            this.createFrame()

        }
        update() {
            super.update()
            if ($gameMessage.isBusy()) {
                this.visible = false
            } else {
                this.visible = true
            }

            //s(param.ClickInvalidSwitchId, !($gameMessage.isBusy() || $osawari.isRanning))

        }
        createFrame() {
            let frame = new PIXI.Graphics();
            frame.lineStyle(0);
            frame.beginFill(FRAME_COLOR);
            frame.drawRoundedRect(0, 0, this.width, this.width, SLOT_RADIUS);
            frame.endFill();
            frame.alpha = FRAME_ALPHA;
            this.frame = this.addChild(frame)
        }
        createForcus() {
            let focus = new PIXI.Graphics();
            focus.lineStyle(LINE_WEIGHT, LINE_COLOR);
            focus.drawRoundedRect(
                Math.floor(0 - LINE_WEIGHT / 2)
                , Math.floor(0 - LINE_WEIGHT / 2)
                , Math.floor(this.width + LINE_WEIGHT / 2)
                , Math.floor(this.width + LINE_WEIGHT / 2)
                , SLOT_RADIUS
            )
            focus.alpha = 1;
            focus.visible = false
            this.focus = this.addChild(focus)
        }
        unset() {
            if (!this.btn) return
            this.removeChild(this.btn)
            this.btn.destroy()
            this.btn = null
        }
        set(classId) {
            this.unset()
            if (!classId) return;

            this.class = $dataClasses[classId]

            this.btn = new Sprite_Clickable()
            this.btn.bitmap = ImageManager.loadPicture("items/icons/" + this.class.name)
            this.btn.onClick = () => { this.use() }
            //let width = this.width
            // this.btn.update = function () {
            //     SpriteBtn.prototype.update.apply(this, arguments)
            //     this.scale.x = this.scale.y = width / this.width
            // }   
            this.addChild(this.btn)
        }

        use() {
            this.parent.children.forEach((a) => {
                a.unuse()
            })
            this.focus.visible = true

            log("slot use: " + this.class.name)
            if (Slot.onUse)
                Slot.onUse(this.class)
        }
        unuse() {
            this.focus.visible = false
        }
    }


    PluginManagerEx.registerCommand(script, "hide", args => {
        $slot.hide()
    });

    PluginManagerEx.registerCommand(script, "show", args => {
        $slot.show()
    });
    PluginManagerEx.registerCommand(script, "set", args => {
        $slot.set(args.slot, args.classId)
    });
    PluginManagerEx.registerCommand(script, "use", args => {
        $slot.use(args.slot)
    });
    PluginManagerEx.registerCommand(script, "clear", args => {
        $slot.clear()
    });


    const SceneManager_onBeforeSceneStart = SceneManager.onBeforeSceneStart
    SceneManager.onBeforeSceneStart = function () {
        if (this._previousScene) {
            this._previousScene.removeChild($slot)
        }
        SceneManager_onBeforeSceneStart.apply(this, arguments);
    }
    const _Scene_Map_start = Scene_Map.prototype.start;
    Scene_Map.prototype.start = function () {
        _Scene_Map_start.apply(this, arguments);
        if ($slot == null)
            $slot = new SlotBar(param.length)

        SceneManager._scene.addChild($slot);
    };







    window.Slot = Slot

})();
